#include <stdio.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include <math.h>
#include <stdlib.h>
#include <stdio.h>


#define ADC_INIT_CHANNEL 0

#define MAX_SPEED 0
#define SPEED_2 2
#define SPEED_4 4
#define SPEED_6 6
#define STOP -1

typedef enum {
	MOTOR_A,
	MOTOR_B,
} Motor;


typedef enum {
	FLOAT,
	FORWARD,
	BACKWARD,
	BREAK,
} Direction;

const int num_channels = 8;

void ADC_init(void) {
	ADMUX = 0x00;
	ADMUX = (1 << REFS0);
	ADCSRA = (1 << ADEN) | (7 << ADPS0);
}

int ADC_get(uint8_t channel)
{
	ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;
	ADCSRA |= (1 << ADSC);
	while(ADCSRA & (1 << ADSC));

	return ADC;
}

/*
 * Functie care initializeaza Timer0
 * in modul Fast PWM, cu top = 0xFF
 * non-inverting si non prescaling
 */
void initPWM(void)
{
	/* setez ca output pinul PB3 --> OC0A */
	DDRB |= (1 << PB3); 
	/* setez ca output pinul PB4 --> OC0B */
	DDRB |= (1 << PB4); 
		
	/* set fast pwm 8-bit, top = 0xFF, modul non-inverting */
	TCCR0A |= (1 << WGM01) | (1 << WGM00) | (1 << COM0A1) | (1 << COM0B1);
	
	/* pentru no prescaling */
	TCCR0B |= (1 << CS00);

}
void setMotorDirection(Motor motor, Direction dir) {
	if(motor == MOTOR_A) {
		/* folosim PORTD -- AIN1 = PB0; AIN2 = PB5 */
		if(dir == FLOAT) {
			PORTB |= (0 << PB0) | (0 << PB5);
		} else if (dir == FORWARD) {
			PORTB |= (0 << PB0) | (1 << PB5);
		} else if (dir == BACKWARD) {
			PORTB |= (1 << PB0) | (0 << PB5);
		} else {
			//BREAK
			PORTB = (1 << PB0) | (1 << PB5);
		}
	} else {
		/* folosim PORTC -- BIN1 = PB1; BIN2 = PB6 */
		if(dir == FLOAT) {
			PORTB |= (0 << PB1) | (0 << PB6);
		} else if (dir == FORWARD) {
			PORTB |= (0 << PB1) | (1 << PB6);
		} else if (dir == BACKWARD) {
			PORTB |= (1 << PB1) | (0 << PB6);
		} else {
			//BREAK
			PORTB = (1 << PB1) | (1 << PB6);
		}
	}
}


void initGPIO(void) {
	/*
	 * Setari pentru senzori
	 */
	/* Setez pinii PA0-PA7 - ca intrare. */
	int i;
	for(i = 0; i < 8; i++) {
		DDRA &= ~(1 << i);
	}
	/* Activez rezistenta de pull-up pentru pinii PA0-PA7. */
	for(i = 0; i < 8; i++) {
		PORTA |= (1 << i);
	}
	
	/*
	 * Setari driver motoare
	 */
	
	/* Setez pinii PB0, PB1, PB5, PB6 - ca iesire. */
	DDRB |= (1 << PB0); //AIN1
	DDRB |= (1 << PB5); //AIN2
	DDRB |= (1 << PB1); //BIN1
	DDRB |= (1 << PB6); //BIN2
		
	/*
	 * Setari LED
	 */
	DDRD |= (1 << PD7);
	PORTD &= ~(1 << PD7);
	
	/* Pun fiecare motor in starea float */
	setMotorDirection(MOTOR_A, FLOAT);
	setMotorDirection(MOTOR_B, FLOAT); 
}

void setMotorPwm(Motor motor, int speed){
	/* Set motor A speed */
	if(motor == MOTOR_A) {
		if(speed == 0)
			OCR0A = 0xFF;
		if(speed == 2)
			OCR0A = 0xFF/2;
		if(speed == 4)
			OCR0A = 0xFF/4;
		if(speed == 6)
			OCR0A = 0xFF/6;
		if(speed == -1)
			OCR0A = 0x00;
	} else { /* Set motor B speed */
		if(speed == 0)
			OCR0B = 0xFF;
		if(speed == 2)
			OCR0B = 0xFF/2;
		if(speed == 4)
			OCR0B = 0xFF/4;
		if(speed == 6)
			OCR0B = 0xFF/6;
		if(speed == -1)
			OCR0B = 0x00;
	}
}

void ledON(void) {
	PORTD |= (1 << PD7);
}

void ledOFF(void){
	PORTD &= ~(1 <<  PD7);
}
int main(void) {
	
	initGPIO();
	
	setMotorDirection(MOTOR_A, FORWARD);
	setMotorDirection(MOTOR_B, FORWARD);
	
	initPWM();
    ADC_init();
	
	/* Get ready */
	_delay_ms(5000);
	while(1){	
	
		/* Start! :) */
		if( ADC_get(0) > 340 && ADC_get(1)  > 340) {
			/* Aprinde LED */
			ledON();
			
			/* Mergi spre stanga */
			setMotorPwm(MOTOR_A, SPEED_4);
			setMotorPwm(MOTOR_B, STOP);
		} else if( ADC_get(6) > 340 && ADC_get(7) > 340) {
			/* Stinge led */
			ledOFF();
			
			/* Mergi spre dreapta */
			setMotorPwm(MOTOR_A, STOP);
			setMotorPwm(MOTOR_B, SPEED_4);
		} else {
			ledOFF();
			/* Mergi inainte */
			setMotorPwm(MOTOR_A, SPEED_4);
			setMotorPwm(MOTOR_B, SPEED_4);
		} 
	}
	return 0;
}



